ifndef MATRIX_H
#define MATRIX_H

#include <iostream>
#include <vector>

namespace std {

class matrix {
public:
    // constructors
    matrix(unsigned int rowNo, unsigned int colNo, double v = 0);
    matrix(const matrix& m);   // copy constructor

    // destructor
    ~matrix(void);
       
    // few useful functions
    matrix & transpose();
    matrix &randomize(unsigned int limit = 10);
    unsigned int getRowNo() const { return rowNo; }
    unsigned int getColNo() const { return colNo; }

    // overloaded self-modifying operators
    matrix& operator= (const matrix& m);
    matrix& operator+= (const matrix& m);
    matrix& operator*= (const matrix& m);
    matrix& operator*= (const double& v);

    // overloaded comparison operators
    // *** it is easier to put them here, as member functions because here 
    // *** private member 'vals' can be accessed to use its comparison operator 
    // *** (defiend in the STL vector class) 
    bool operator== (const matrix & m) const {return (*vals == *m.vals);}
    bool operator!= (const matrix & m) const {return !operator==(m);}

    // overloaded subscript operators (for modifying & access)
    double& operator()(unsigned int row, unsigned int col);
    double  operator()(unsigned int row, unsigned int col) const;

private:
    unsigned int rowNo,colNo;
    
    // a pointer to a STL vector of STL vectors of doubles:
    vector<vector<double> > * vals;   // notice the space between both '>' signs
    
    void error(const char* pMsg) const;
};

// ******************* OVERLOADED NON-MODYFYING OPERATORS *******************

matrix operator+ (const matrix & m1, const matrix & m2);
matrix operator* (const matrix & m, const double & v);
matrix operator* (const double & v, const matrix & m);
matrix operator* (const matrix & m1, const matrix & m2);
ostream & operator << (ostream & ostr, const matrix & m);

} // namespace std

#endif // MATRIX_H
